@@ -15,7 +15,7 @@
 # ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 # OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 
-from typing import Optional, Union, Match, Any, Pattern, NoReturn, List, Dict
+from typing import Optional, Union, Match, Any, Pattern, NoReturn, List, Dict, cast
 
 import sys, os, io, importlib.util, argparse, re, getpass, datetime, traceback
 import jinja2, jinja2.sandbox, yaml, pytz
@@ -29,8 +29,7 @@ from cryptography.hazmat.primitives.ciphers.algorithms import AES
 from cryptography.hazmat.primitives.ciphers.modes import CTR
 from cryptography.exceptions import InvalidSignature
 
-
-__version__ = '1.16.0'
+__version__ = '1.16.2'
 
 def main() -> None:
   exc_source = None
@@ -434,10 +433,10 @@ class __ArgumentParser(argparse.ArgumentParser):
 
 class JinjaFx():
   def jinjafx(self, template: Union[str, io.TextIOWrapper], data: Optional[str], gvars: Dict[str, Any], output: str, exts_dirs: Optional[List[str]]=None, sandbox: bool=False) -> Dict[str, List[str]]:
-    self.__g_datarows: List[List[str]] = []
-    self.__g_dict: Dict[str, int] = {}
+    self.__g_datarows: List[List[Union[str, int, float]]] = []
+    self.__g_dict: Dict[str, Union[int, str, bool]] = {}
     self.__g_row = 0 
-    self.__g_vars: Dict[str, Any] = {}
+    self.__g_vars: Dict[str, Union[int, str, float, bool]] = {}
     self.__g_warnings: List[str] = []
     self.__g_xlimit = 5000 if sandbox else 0
 
@@ -528,7 +527,7 @@ class JinjaFx():
                 ufields.append(re.sub(r'^(["\'])(.*)\1$', r'\2', f))
   
               n = len(self.__g_datarows[0])
-              fields = [list(map(self.__jfx_expand, ufields[:n] + [''] * (n - len(ufields)), [True] * n))]
+              fields = cast(List[List[Union[int, str, float, List[str], List[List[str]]]]], [list(map(self.__jfx_expand, ufields[:n] + [''] * (n - len(ufields)), [True] * n))])
 
               row = 0
               while fields:
@@ -536,12 +535,12 @@ class JinjaFx():
                   fields[0].insert(0, rowkey)
                   rowkey += 1
   
-                if any(isinstance(col[0], list) for col in fields[0][1:]):
+                if any(isinstance(colx[0], list) for colx in cast(List[Any], fields[0][1:])):
                   for col in range(1, len(fields[0])):
-                    if isinstance(fields[0][col][0], list):
-                      for i, v in enumerate(fields[0][col][0]):
+                    if isinstance(cast(List[Any], fields[0][col])[0], list):
+                      for i, v in enumerate(cast(List[Any], fields[0][col])[0]):
                         nrow = [e[:] if isinstance(e, list) else e for e in fields[0]]
-                        nrow[col] = [v, fields[0][col][1][i]]
+                        nrow[col] = [v, cast(List[Any], fields[0][col])[1][i]]
                         fields.append(nrow)
   
                       fields.pop(0)
@@ -551,45 +550,45 @@ class JinjaFx():
                   xgroups: List[List[str]] = []
   
                   for col in range(1, len(fields[0])):
-                    fields[0][col][0] = recm.sub(lambda m: self.__jfx_data_counter(m, fields[0][0], col, row), fields[0][col][0])
+                    cast(List[Any], fields[0][col])[0] = recm.sub(lambda m: self.__jfx_data_counter(m, cast(int, fields[0][0]), col, row), cast(List[Any], fields[0][col])[0])
   
-                    for g in range(len(fields[0][col][1])):
-                      fields[0][col][1][g] = recm.sub(lambda m: self.__jfx_data_counter(m, fields[0][0], col, row), fields[0][col][1][g])
+                    for g in range(len(cast(List[Any], fields[0][col])[1])):
+                      cast(List[Any], fields[0][col])[1][g] = recm.sub(lambda m: self.__jfx_data_counter(m, cast(int, fields[0][0]), col, row), cast(List[Any], fields[0][col])[1][g])
   
-                    xgroups.append(fields[0][col][1])
+                    xgroups.append(cast(List[Any], fields[0][col])[1])
  
                   groups = dict(enumerate(sum(xgroups, ['\\0'])))
   
                   for col in range(1, len(fields[0])):
-                    fields[0][col] = re.sub(r'\\([0-9]+)', lambda m: groups.get(int(m.group(1)), '\\' + str(m.group(1))), fields[0][col][0])
+                    fields[0][col] = re.sub(r'\\([0-9]+)', lambda m: groups.get(int(m.group(1)), '\\' + str(m.group(1))), cast(List[Any], fields[0][col])[0])
   
                     delta = 0
-                    for m in re.finditer(r'([0-9]+)(?<!\\)\%([0-9]+)', fields[0][col]):
+                    for m in re.finditer(r'([0-9]+)(?<!\\)\%([0-9]+)', cast(str, fields[0][col])):
                       pvalue = str(int(m.group(1))).zfill(int(m.group(2)))
-                      fields[0][col] = fields[0][col][:m.start() + delta] + pvalue + fields[0][col][m.end() + delta:]
+                      fields[0][col] = cast(str, fields[0][col])[:m.start() + delta] + pvalue + cast(str, fields[0][col])[m.end() + delta:]
   
                       if len(m.group(0)) > len(pvalue):
                         delta -= len(m.group(0)) - len(pvalue)
                       else:
                         delta += len(pvalue) - len(m.group(0))
   
-                    fields[0][col] = re.sub(r'\\([}{%])', r'\1', fields[0][col])
+                    fields[0][col] = re.sub(r'\\([}{%])', r'\1', cast(str, fields[0][col]))
   
                     if col in int_indices:
-                      fields[0][col] = int(fields[0][col])
+                      fields[0][col] = int(cast(str, fields[0][col]))
 
                     elif col in float_indices:
-                      fields[0][col] = float(fields[0][col])
+                      fields[0][col] = float(cast(str, fields[0][col]))
 
                   include_row = True
                   if jinjafx_filter:
                     for index in jinjafx_filter:
-                      if not re.search(jinjafx_filter[index], fields[0][index]):
+                      if not re.search(jinjafx_filter[index], cast(str, fields[0][index])):
                         include_row = False
                         break
   
                   if include_row:
-                    self.__g_datarows.append(fields[0])
+                    self.__g_datarows.append(cast(List[Union[str, int, float]], fields[0]))
   
                   fields.pop(0)
                   row += 1
@@ -607,7 +606,7 @@ class JinjaFx():
           for rx, v in field[fn].items():
             mv.append([re.compile(rx + '$'), v])
 
-          self.__g_datarows[1:] = sorted(self.__g_datarows[1:], key=lambda n: (self.__find_re_match(mv, n[self.__g_datarows[0].index(fn.lstrip('+-')) + 1]), n[self.__g_datarows[0].index(fn.lstrip('+-')) + 1]), reverse=r)
+          self.__g_datarows[1:] = sorted(self.__g_datarows[1:], key=lambda n: (self.__find_re_match(mv, cast(str, n[self.__g_datarows[0].index(fn.lstrip('+-')) + 1])), n[self.__g_datarows[0].index(fn.lstrip('+-')) + 1]), reverse=r)
 
         else:
           r = True if field.startswith('-') else False
@@ -678,11 +677,11 @@ class JinjaFx():
     routput = env.from_string(output)
 
     for row in range(1, max(2, len(self.__g_datarows))):
-      rowdata: Dict[str, str] = {}
+      rowdata: Dict[str, Union[str, int, float]] = {}
 
       if self.__g_datarows:
         for col in range(len(self.__g_datarows[0])):
-          rowdata.update({ self.__g_datarows[0][col]: self.__g_datarows[row][col + 1] })
+          rowdata.update({ cast(str, self.__g_datarows[0][col]): self.__g_datarows[row][col + 1] })
 
         env.globals['jinjafx'].update({ 'row': row })
         self.__g_row = row
@@ -795,7 +794,7 @@ class JinjaFx():
     return default
 
 
-  def __jfx_lookup(self, method: str, *args: str) -> Any:
+  def __jfx_lookup(self, method: str, *args: str) -> Union[Union[int, str, float, bool], List[str]]:
     if method == 'vars' or method == 'ansible.builtin.vars':
       if args:
         default: Optional[str] = args[1] if len(args) > 1 else None
@@ -846,12 +845,12 @@ class JinjaFx():
 
     if self.__g_dict.get(key + '_' + str(row), True):
       n = self.__g_dict.get(key, int(start) - int(increment))
-      self.__g_dict[key] = n + int(increment)
+      self.__g_dict[key] = int(n) + int(increment)
       self.__g_dict[key + '_' + str(row)] = False
     return str(self.__g_dict[key])
 
 
-  def __jfx_expand(self, s: str, rg: bool=False) -> Any:
+  def __jfx_expand(self, s: str, rg: bool=False) -> Union[List[str], List[Union[List[str], List[List[str]]]]]:
     pofa: List[str] = [s]
     groups: List[List[str]] = [[s]]
 
@@ -955,7 +954,12 @@ class JinjaFx():
       g.pop(0)
 
     pofa = [re.sub(r'\\([\|\(\[\)\]])', r'\1', i) for i in pofa]
-    return [pofa, groups] if rg else pofa
+
+    if rg:
+      ret: List[Union[List[str], List[List[str]]]] = [pofa, groups]
+      return ret
+ 
+    return pofa
 
 
   def __jfx_fandl(self, forl: str, fields: Optional[List[str]], ffilter: Dict[str, str]) -> bool:
@@ -1021,7 +1025,7 @@ class JinjaFx():
     return self.__jfx_fandl('last', fields, ffilter)
 
   
-  def __jfx_fields(self, field: Optional[str]=None, ffilter: Dict[str, str]={}) -> Optional[List[str]]:
+  def __jfx_fields(self, field: Optional[str]=None, ffilter: Dict[str, str]={}) -> Optional[List[Union[str, int, float]]]:
     if field is not None:
       if field in self.__g_datarows[0]:
         fpos = self.__g_datarows[0].index(field) + 1
@@ -1030,7 +1034,7 @@ class JinjaFx():
     else:
       return None
     
-    field_values: List[str] = []
+    field_values: List[Union[str, int, float]] = []
         
     for r in range(1, len(self.__g_datarows)):
       fmatch = True
@@ -1054,7 +1058,7 @@ class JinjaFx():
     return field_values
 
  
-  def __jfx_data(self, row: int, col: Optional[Union[str, int]]=None) -> Optional[Union[str, List[str]]]:
+  def __jfx_data(self, row: int, col: Optional[Union[str, int]]=None) -> Optional[Union[Union[str, int, float], List[Union[str, int, float]]]]:
     if self.__g_datarows:
       if isinstance(col, str):
         if col in self.__g_datarows[0]:
@@ -1093,16 +1097,16 @@ class JinjaFx():
       key = '_cnt_k_' + str(key)
 
     n = self.__g_dict.get(key, int(start) - int(increment))
-    self.__g_dict[key] = n + int(increment)
-    return self.__g_dict[key]
+    self.__g_dict[key] = int(n) + int(increment)
+    return int(self.__g_dict[key])
 
 
-  def __jfx_setg(self, key: str, value: Any) -> str:
+  def __jfx_setg(self, key: str, value: Union[int, str, bool]) -> str:
     self.__g_dict['_val_' + str(key)] = value
     return ''
 
 
-  def __jfx_getg(self, key: str, default: Optional[str]=None) -> Any:
+  def __jfx_getg(self, key: str, default: Optional[Union[int, str, bool]]=None) -> Optional[Union[int, str, bool]]:
     return self.__g_dict.get('_val_' + str(key), default)
 
 
